@@ -46,6 +46,13 @@ module Numeric.LinearAlgebra.Static(
     matrix,
     -- * Complex
     ℂ, C, M, Her, her, 𝑖,
+    toComplex,
+    fromComplex,
+    complex,
+    real,
+    imag,
+    sqMagnitude,
+    magnitude,
     -- * Products
     (<>),(#>),(<.>),
     -- * Linear Systems
@@ -74,7 +81,9 @@ import Numeric.LinearAlgebra hiding (
     (<\>),fromList,takeDiag,svd,eig,eigSH,
     eigenvalues,eigenvaluesSH,build,
     qr,size,dot,chol,range,R,C,sym,mTm,unSym,
-    randomVector,rand,randn,gaussianSample,uniformSample,meanCov)
+    randomVector,rand,randn,gaussianSample,uniformSample,meanCov,
+    toComplex, fromComplex, complex, real, magnitude
+    )
 import qualified Numeric.LinearAlgebra as LA
 import qualified Numeric.LinearAlgebra.Devel as LA
 import Data.Proxy(Proxy(..))
@@ -243,6 +252,29 @@ instance KnownNat n => Diag (M n n) (C n)
   where
     takeDiag x = mkC (LA.takeDiag (extract x))
 
+--------------------------------------------------------------------------------
+
+
+toComplex :: KnownNat n => (R n, R n) -> C n
+toComplex (r,i) = mkC $ LA.toComplex (ud1 r, ud1 i)
+
+fromComplex :: KnownNat n => C n -> (R n, R n)
+fromComplex (C (Dim v)) = let (r,i) = LA.fromComplex v in (mkR r, mkR i)
+
+complex :: KnownNat n => R n -> C n
+complex r = mkC $ LA.toComplex (ud1 r, LA.konst 0 (size r))
+
+real :: KnownNat n => C n -> R n
+real = fst . fromComplex
+
+imag :: KnownNat n => C n -> R n 
+imag = snd . fromComplex
+
+sqMagnitude :: KnownNat n => C n -> R n
+sqMagnitude c = let (r,i) = fromComplex c in r**2 + i**2
+
+magnitude :: KnownNat n => C n -> R n
+magnitude = sqrt . sqMagnitude
 
 --------------------------------------------------------------------------------
 
@@ -383,7 +415,7 @@ split (extract -> v) = ( mkR (subVector 0 p' v) ,
 
 
 headTail :: (KnownNat n, 1<=n) => R n -> (ℝ, R (n-1))
-headTail = ((!0) . extract *** id) . split
+headTail = ((! 0) . extract *** id) . split
 
 
 splitRows :: forall p m n . (KnownNat p, KnownNat m, KnownNat n, p<=m) => L m n -> (L p n, L (m-p) n)
@@ -595,9 +627,9 @@ mulR (isDiag -> Just (0,a,_)) (isDiag -> Just (0,b,_)) = diagR 0 (mkR v :: R k)
     a' = subVector 0 n a
     b' = subVector 0 n b
 
-mulR (isDiag -> Just (0,a,_)) (extract -> b) = mkL (asColumn a * takeRows (LA.size a) b)
+-- mulR (isDiag -> Just (0,a,_)) (extract -> b) = mkL (asColumn a * takeRows (LA.size a) b)
 
-mulR (extract -> a) (isDiag -> Just (0,b,_)) = mkL (takeColumns (LA.size b) a * asRow b)
+-- mulR (extract -> a) (isDiag -> Just (0,b,_)) = mkL (takeColumns (LA.size b) a * asRow b)
 
 mulR a b = mkL (extract a LA.<> extract b)
 
@@ -658,9 +690,9 @@ mulC (isDiagC -> Just (0,a,_)) (isDiagC -> Just (0,b,_)) = diagR 0 (mkC v :: C k
     a' = subVector 0 n a
     b' = subVector 0 n b
 
-mulC (isDiagC -> Just (0,a,_)) (extract -> b) = mkM (asColumn a * takeRows (LA.size a) b)
+-- mulC (isDiagC -> Just (0,a,_)) (extract -> b) = mkM (asColumn a * takeRows (LA.size a) b)
 
-mulC (extract -> a) (isDiagC -> Just (0,b,_)) = mkM (takeColumns (LA.size b) a * asRow b)
+-- mulC (extract -> a) (isDiagC -> Just (0,b,_)) = mkM (takeColumns (LA.size b) a * asRow b)
 
 mulC a b = mkM (extract a LA.<> extract b)
 
